var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import last from 'lodash/last';
import trim from 'lodash/trim';
import some from 'lodash/some';
import { STSClient, GetCallerIdentityCommand, STSServiceException } from '@aws-sdk/client-sts';
import { IAMClient, IAMServiceException, ListAttachedUserPoliciesCommand, } from '@aws-sdk/client-iam';
export var CredentialsValidationErrorCode;
(function (CredentialsValidationErrorCode) {
    CredentialsValidationErrorCode["NotAdmin"] = "NOT_ADMIN";
    CredentialsValidationErrorCode["InvalidKeys"] = "INVALID_KEYS";
    CredentialsValidationErrorCode["AccountMismatch"] = "ACCOUNT_MISMATCH";
    CredentialsValidationErrorCode["Unknown"] = "UNKNOWN";
})(CredentialsValidationErrorCode || (CredentialsValidationErrorCode = {}));
export class CredentialsValidationException extends Error {
    constructor(code, rootCause) {
        super(rootCause ? `${code} - rootCause.message` : `${code}`);
        this.name = 'ValidationError';
        this.code = code;
        this.rootCause = rootCause;
    }
}
/**
 * Validates if the credentials are for an admin. If not, an exception is thrown.
 * Validating the credentials includes the following steps:
 * - Get the account id and user arn by calling caller identity
 * - Get the managed policies attached to the principal and ensure that one of them is AdministratorAccess
 *
 * If all checks out, this function returns the account id and the user name
 */
export function validateCredentials({ accessKey, secretKey, region, }) {
    return __awaiter(this, void 0, void 0, function* () {
        accessKey = trim(accessKey);
        secretKey = trim(secretKey);
        const stsClient = new STSClient({
            region,
            credentials: {
                accessKeyId: accessKey,
                secretAccessKey: secretKey,
            },
        });
        const iamClient = new IAMClient({
            region,
            credentials: {
                accessKeyId: accessKey,
                secretAccessKey: secretKey,
            },
        });
        let userName;
        let accountId;
        let arn;
        try {
            // Get the account id and the user arn
            const stsData = yield stsClient.send(new GetCallerIdentityCommand({}));
            const { Account, Arn } = stsData;
            accountId = Account;
            arn = Arn;
        }
        catch (err) {
            let code = CredentialsValidationErrorCode.Unknown;
            // InvalidClientTokenId => incorrectly copied, or inactive or deleted
            if (err instanceof STSServiceException) {
                if (err.name === 'InvalidClientTokenId' || err.name === 'SignatureDoesNotMatch')
                    code = CredentialsValidationErrorCode.InvalidKeys;
            }
            throw new CredentialsValidationException(code, err);
        }
        if (arn === `arn:aws:iam::${accountId}:root`) {
            // For now, we consider the root credentials as valid, although it is not a good idea for the
            // user to provide root credentials
            return { accountId, userArn: arn };
        }
        if (!(arn === null || arn === void 0 ? void 0 : arn.startsWith(`arn:aws:iam::${accountId}:user/`))) {
            // We don't know what the principal is so we consider the credentials invalid
            throw new CredentialsValidationException(CredentialsValidationErrorCode.Unknown);
        }
        try {
            // Get the managed policies attached to the principal and ensure that one of them is AdministratorAccess
            userName = last(arn.split('/'));
            const params = {
                UserName: userName,
                MaxItems: 999,
            };
            let found = false;
            let iamData;
            do {
                iamData = yield iamClient.send(new ListAttachedUserPoliciesCommand(params));
                found = some(iamData.AttachedPolicies, {
                    PolicyArn: 'arn:aws:iam::aws:policy/AdministratorAccess',
                });
                params.Marker = iamData.Marker;
            } while (!found && iamData.IsTruncated);
            if (!found) {
                throw new CredentialsValidationException(CredentialsValidationErrorCode.NotAdmin);
            }
            return { accountId, userArn: arn, userName };
        }
        catch (err) {
            let code = CredentialsValidationErrorCode.Unknown;
            if (err instanceof IAMServiceException) {
                if (err.name === 'AccessDenied')
                    code = CredentialsValidationErrorCode.NotAdmin;
            }
            if (err instanceof CredentialsValidationException) {
                throw err;
            }
            throw new CredentialsValidationException(code, err);
        }
    });
}
//# sourceMappingURL=validate-credentials.js.map